# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import instances.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Instance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, verbose_name='ID', serialize=False)),
                ('label', instances.fields.DNSLabelField(unique=True, max_length=63, db_index=True)),
                ('title', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True)),
                ('created_by', models.ForeignKey(related_name='created_instances', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('users', models.ManyToManyField(related_name='instances', to=settings.AUTH_USER_MODEL, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import instances.fields


class Migration(migrations.Migration):

    dependencies = [
        ('instances', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='instance',
            options={'verbose_name': 'instance', 'verbose_name_plural': 'instances'},
        ),
        migrations.AlterField(
            model_name='instance',
            name='created_by',
            field=models.ForeignKey(related_name='created_instances', verbose_name='created by', blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AlterField(
            model_name='instance',
            name='description',
            field=models.TextField(verbose_name='description', blank=True),
        ),
        migrations.AlterField(
            model_name='instance',
            name='label',
            field=instances.fields.DNSLabelField(unique=True, max_length=63, verbose_name='label', db_index=True),
        ),
        migrations.AlterField(
            model_name='instance',
            name='title',
            field=models.CharField(max_length=100, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='instance',
            name='users',
            field=models.ManyToManyField(related_name='instances', verbose_name='users', to=settings.AUTH_USER_MODEL, blank=True),
        ),
    ]



import re

from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.cache import patch_vary_headers

from .models import Instance


class MultiInstanceMiddleware:
    """
    Check for a hostname of the form <instance>.BASE_HOST, or <instance>.127.0.0.1.nip.io;
    if not given, use ROOT_URLCONF_HOST or instances.urls as the urlconf. If given, check
    if it exists in the database; if not, redirect to BASE_HOST, if it does,
    set request.instance and check if the logged in user has access to it
    """
    def process_request(self, request):
        host = request.get_host().lower()
        domain = getattr(settings, 'BASE_HOST', '127.0.0.1.nip.io')
        pattern = r'^(?P<instance>.*?)\.%s(?::(?P<port>.*))?$' % re.escape(domain)
        matches = re.match(pattern, host)
        if not matches:
            request.instance = None
            request.urlconf = getattr(settings, 'ROOT_URLCONF_HOST', 'instances.urls')
            return

        try:
            request.instance = Instance.objects.get(label=matches.group('instance'))
        except:
            url = '%(scheme)s://%(domain)s' % {
                'scheme': 'https' if request.is_secure() else 'http',
                'domain': domain,
                }
            if matches.group('port'):
                url += ':' + matches.group('port')
            return HttpResponseRedirect(url)

        request.is_user_instance = request.user.is_authenticated() and (
            request.instance in request.user.instances.all() or request.user.is_superuser)

    def process_response(self, request, response):
        patch_vary_headers(response, ('Host',))
        return response



import re

from django.db import models
from django.core.validators import RegexValidator
from django.utils.translation import ugettext_lazy

label_re = re.compile(r'(?i)^[a-z0-9][a-z0-9-]*[a-z0-9]$')
validate_label = RegexValidator(
    label_re, ugettext_lazy("Enter a valid instance label consisting of letters, numbers, or hyphens."), 'invalid')


class Creator(object):
    def __init__(self, field):
        self.field = field

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        return obj.__dict__[self.field.name]

    def __set__(self, obj, value):
        obj.__dict__[self.field.name] = self.field.to_python(value)


class DNSLabelField(models.CharField):
    description = "A DNS label"

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 63)
        super(DNSLabelField, self).__init__(*args, **kwargs)
        self.validators.append(validate_label)

    def contribute_to_class(self, cls, name):
        super(DNSLabelField, self).contribute_to_class(cls, name)
        setattr(cls, name, Creator(self))

    def to_python(self, value):
        value = super(DNSLabelField, self).to_python(value)
        if value == '' or value is None:
            return value
        # Want immediate validation on any instance creation
        validate_label(value)
        return value.lower()



from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .fields import DNSLabelField


class InstanceManager(models.Manager):
    def for_instance(self, instance):
        return self.get_queryset().filter(instance=instance)


@python_2_unicode_compatible
class Instance(models.Model):
    label = DNSLabelField(_('label'), db_index=True, unique=True)
    title = models.CharField(_('title'), max_length=100)
    description = models.TextField(_('description'), blank=True)
    users = models.ManyToManyField(
        User, verbose_name=_('users'), related_name='instances', blank=True)
    created_by = models.ForeignKey(
        User, verbose_name=_('created by'), related_name='created_instances',
        null=True, blank=True)

    class Meta:
        verbose_name = _('instance')
        verbose_name_plural = _('instances')

    def __str__(self):
        return u'%s (%s)' % (self.title, self.label)

    def get_absolute_url(self):
        url = 'http://%s.%s' % (self.label, getattr(settings, 'BASE_HOST', '127.0.0.1.nip.io'))
        if getattr(settings, 'BASE_PORT', None):
            url += ':%s' % settings.BASE_PORT
        return url


class InstanceMixin(models.Model):
    instance = models.ForeignKey(Instance, verbose_name=_('instance'))

    objects = InstanceManager()

    class Meta:
        abstract = True



from django.conf.urls import url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.generic import ListView

from instances.models import Instance

urlpatterns = [
    url(r'^$', ListView.as_view(
        queryset=Instance.objects.all(),
        context_object_name='instances',
        template_name='instances/index.html',
    )),
]

urlpatterns += staticfiles_urlpatterns()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os

file_dir = os.path.abspath(os.path.dirname(__file__))


def read_file(filename):
    filepath = os.path.join(file_dir, filename)
    return open(filepath).read()

setup(
    name="django-subdomain-instances",
    version='2.0',
    description='A way of allowing subdomains to be served by the same project, '
                'and associating objects with particular subdomains.',
    long_description=read_file('README.rst'),
    author='mySociety',
    author_email='matthew@mysociety.org',
    url='https://github.com/mysociety/django-subdomain-instances',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django >=1.8.6,<2.0',
    ],
    classifiers=[
        'Framework :: Django',
    ],
)
